@@ -32,12 +32,13 @@ except ImportError:
 class open(OpenBase):
     def __init__(self, filename="", flags=[], radare2home=None):
         super(open, self).__init__(filename, flags)
+        self.pending = b''
         if filename.startswith("http://"):
             self._cmd = self._cmd_http
             self.uri = filename + "/cmd"
         elif filename.startswith("ccall://"):
             self._cmd = self._cmd_native
-            self.uri = filename[7:]
+            self.uri = filename[8:]
         elif filename.startswith("tcp://"):
             r = re.match(r"tcp://(\d+\.\d+.\d+.\d+):(\d+)/?", filename)
             if not r:
@@ -66,7 +67,17 @@ class open(OpenBase):
                 )
             except:
                 raise Exception("ERROR: Cannot find radare2 in PATH")
-            self.process.stdout.read(1)  # Reads initial \x00
+            hello_cmd = True
+            if hello_cmd:
+                self.process.stdout.read(1)  # Reads initial \x00
+                try:
+                    self.process.stdin.write(("?V\n").encode("utf8"))
+                    self.process.stdin.flush()
+                    r = self.process.stdout
+                    r.read(4096)
+                except:
+                    raise Exception("ERROR: Cannot open %s" % filename)
+            
             # make it non-blocking to speedup reading
             self.nonblocking = True
             if self.nonblocking:
@@ -106,29 +117,39 @@ class open(OpenBase):
 
     def _cmd_process(self, cmd):
         cmd = cmd.strip().replace("\n", ";")
-        self.process.stdin.write((cmd + "\n").encode("utf8"))
+        try:
+            self.process.stdin.write((cmd + "\n").encode("utf8"))
+        except:
+            return ''
         r = self.process.stdout
         self.process.stdin.flush()
         out = b""
+        foo = None
         while True:
             if self.nonblocking:
                 try:
-                    foo = r.read(4096)
+                    if len(self.pending) > 0:
+                        foo = self.pending
+                        self.pending = b""
+                    else:
+                        foo = r.read(4096)
+                    if foo is not None:
+                        zro = foo.find(b"\x00")
+                        if zro != -1:
+                            out += foo[0:zro]
+                            if zro < len(foo):
+                                self.pending = foo[zro + 1:]
+                            break
+                        out += foo
                 except:
-                    continue
+                    pass
             else:
                 foo = r.read(1)
-            if foo:
-                if foo.endswith(b"\0"):
-                    out += foo[:-1]
-                    break
-
-                out += foo
-            else:
-                # if there is no any output from pipe this loop will eat CPU, probably we have to do micro-sleep here
-                if self.nonblocking:
-                    time.sleep(0.001)
-
+                if foo is not None:
+                    if foo == b"\x00":
+                        break
+                    out += foo
+            time.sleep(0.001)
         return out.decode("utf-8", errors="ignore")
 
     def _cmd_http(self, cmd):
